#include <stdio.h>
#include <io.h>
#include <string.h>
#include "psftag.h"

// generate a batch file for renaming PSFs based on a template and a filespec

int getfiletitle(char *filename, char *title, char * titlefmt);

int main(int argc, char ** argv) {
    struct _finddata_t fileinfo;
    long searchhandle;
    char buf[1025];

    if (argc != 3) {
        printf("usage: %s \"%%game%% - %%title%%\" filespec",argv[0]);
        return 1;
    }

    if ((searchhandle=_findfirst(argv[2],&fileinfo)) < 0) {
        printf("%s not found\n",argv[2]);
        return 1;
    }
    printf("REM PSF Renamer 1.0 (template: \"%s\" filespec: \"%s\")\n",argv[1],argv[2]);
    do {
        if (getfiletitle(fileinfo.name,buf,argv[1])) printf("REM failed to read tags in %s\n",fileinfo.name);
        else {
            //printf("REM rename %s to %s\n",fileinfo.name,buf);
            //rename(fileinfo.name,buf);
            printf("ren \"%s\" \"%s\"\n",fileinfo.name,buf);
        }
    } while (!_findnext(searchhandle,&fileinfo));

    _findclose(searchhandle);

    return 0;
}

int getfiletitle(char *filename, char *title, char * titlefmt)
{
    char *psftag,tempchar;
    FILE * hFile;
    unsigned long reservesize,FileSize;
    char buf[10];
    char formatname[128];
    unsigned long taglen,tagstart;
    int c,tagnamestart;
    char buffer[1025];

    hFile=fopen(filename,"rb");

    fread(buf,1,4,hFile); // read PSF header

    if (strstr(buf,"PSF") != buf) {printf("REM no PSF header located\n"); return 1;}

    switch (buf[3]) {
    case 0x01:
        sprintf(formatname,"Playstation (PSF1)");
        break;
    case 0x02:
        sprintf(formatname,"Playstation 2 (PSF2)");
        break;
    case 0x11:
        sprintf(formatname,"Saturn (SSF)");
        break;
    case 0x12:
        sprintf(formatname,"Dreamcast (DSF)");
        break;
    case 0x21:
        sprintf(formatname,"Nintendo 64 (USF)");
        break;
    case 0x41:
        sprintf(formatname,"Capcom QSound (QSF)");
        break;
    default:
        sprintf(formatname,"Unknown PSF subformat (0x%02x)",buf[3]);
        break;
    }

    fread(&reservesize,4,1,hFile); // size of reserved area
    fread(&FileSize,4,1,hFile); // size of program

    // seek to where tags should be

    tagstart=reservesize+FileSize+16;
    fseek(hFile,tagstart,SEEK_SET);

    if (fread(buf,1,5,hFile) != 5) {printf("REM no [TAG] 1\n"); return 1;}

    if (strstr(buf,"[TAG]") != buf) {printf("REM no [TAG] 2\n"); return 1;}

    fseek(hFile,0,SEEK_END);
    taglen=ftell(hFile)-tagstart-5;

    psftag=(char *)malloc(taglen+2);
				
    fseek(hFile,tagstart+5,SEEK_SET);

    fread(psftag,taglen,1,hFile);

    psftag[taglen]='\0';

    fclose(hFile);

    // create title from PSF tag
    sprintf(title,"");
    c=0;
    while (c < strlen(titlefmt)) {
        if (titlefmt[c]=='%') {
            tagnamestart=++c;
            for (;c<strlen(titlefmt) && titlefmt[c] != '%'; c++);
            if (c == strlen(titlefmt)) {
                printf("REM Bad title format string (unterminated token)\n");
                return 1;
            }
            titlefmt[c]='\0';
            psftag_raw_getvar(psftag,titlefmt+tagnamestart,buffer,sizeof(buffer)-1);
            strcat(title,buffer);
            if (strlen(buffer)==0 ) printf("REM %s tag missing\n",titlefmt+tagnamestart);
            titlefmt[c]='%';
        } else {
            tempchar=titlefmt[c+1];
            titlefmt[c+1]='\0';
            strcat(title,titlefmt+c);
            titlefmt[c+1]=tempchar;
        }
        c++;
    }

    free(psftag);
    psftag=NULL;

    return 0;
}
